/*
 * DTL_AFMInterpolator.java
 *  Version 1.1
 *  Koen Mostert
 *  November 14, 2007
 *  Copyright (c) Dutch Type Library, 2007. All rights reserved
 *
 * java AFMInterpolator [file1] [file2] [factor] [flag] [output] ([limit])
 *  - file1: first AFM file
 *  - file2: second AFM file
 *  - factor: interpolation percentage (0-100)
 *  - flag: 0 leaves incomplete pairs out, 1 ignores missing pairs and calculates using 0 instead
 *  - output: output file
 *  - limit: lower limit for values, leaves pairs with value <= limit out (optional)
 *
 * example use:
 *  java AFMInterpolator fontname_regular.afm fontname_bold.afm 50 1 fontname_medium.afm 10
 *
 */

import java.io.*;
import java.util.HashMap;

public class AFMInterpolator {
  public static void main(String[] args) throws IOException {

    // get interpolation factor, handling flag and lower limit
    int factor = Integer.parseInt(args[2]);
    int flag = Integer.parseInt(args[3]);
    int limit = (args.length > 5) ? Integer.parseInt(args[5]) : 0;
    
    // create HashMap for storing all pairs in file1
    HashMap<String,Integer> h = new HashMap<String,Integer>();

    // open and read first line of file1
    BufferedReader file1 = new BufferedReader(new FileReader(args[0]));
    String line = file1.readLine();

    // loop through file1
    while (line != null) {
      if (line.length() > 3 && (line.substring(0, 3).equals("KPX") || line.substring(0, 3).equals("POS"))) { // if line starts with "KPX" or "POS"
	String[] values = line.split("\\s+");
	h.put(values[1] + " " + values[2], Integer.parseInt(values[3])); // store pair in the HashMap with (pair, value)
      }
      line = file1.readLine();
    }
    file1.close();

    // open output file to write
    BufferedWriter out = new BufferedWriter(new FileWriter(args[4]));

    // open and read first line of file2
    BufferedReader file2 = new BufferedReader(new FileReader(args[1]));
    line = file2.readLine();

    // loop through file2
    while (line != null) {
      if (line.length() > 3 && (line.substring(0, 3).equals("KPX") || line.substring(0, 3).equals("POS"))) { // if line starts with "KPX" or "POS"
	String[] values = line.split("\\s+");

	Integer first = h.remove(values[1] + " " + values[2]); // remove pair from HashMap

	if (first == null && flag == 1) first = 0; // if the pair didn't exist and flag is set, set first to 0

	if (first != null) {
	  int interpolated = ( first * factor + Integer.parseInt(values[3]) * (100-factor) ) / 100; // interpolated value = (pair1 * factor + pair2 * (100-factor) / 100
	  if (Math.abs(interpolated) > limit) // if bigger than lower limit
	    out.write("KPX " + values[1] + " " + values[2] + " " + interpolated + "\n"); // output new value to file
	}
      }
      line = file2.readLine();
    }

    // if flag is set, loop through leftover values of file1
    if (flag == 1 && h.size() > 0) {
      Object[] leftover = h.keySet().toArray();

      for (int i = 0; i < leftover.length; i++) {
	Integer value = h.remove(leftover[i]);
	int interpolated = (value * factor) / 100;
	if (Math.abs(interpolated) > limit) // if bigger than lower limit
	  out.write("KPX " + leftover[i] + " " + interpolated + "\n");
      }
    }
    out.close();
  }
}
